-- AVERTISSEMENTS:
-- Cette description ne peut en aucun cas tre utilis  des fins lucratives
-- (sauf autorisation crite).
-- Je me dgage de toute responsabilit quant aux consquences lies 
-- l'utilisation du contenu de ce fichier.
-- Tous droits reserves - Steve MULLER 2007
--------------------------------------------------------------------------------
----------------- Ceci est la description d'un compteur 16 bit. ----------------
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity cnt16 is
  port (clr : in  std_logic;
        clk : in  std_logic;
        q   : out std_logic_vector(15 downto 0));
end cnt16;

architecture behavioral of cnt16 is

  signal internal : std_logic_vector(15 downto 0);

  begin
    process (clr, clk)
    begin
      if (clr = '1') then
        internal <= (others => '0');
      elsif (rising_edge(clk)) then
        internal <= internal + 1;
      end if;
    end process;
    q <= internal;
  end behavioral;
--------------------------------------------------------------------------------
---------------- Ceci est la description d'un bascule RS clock. ---------------
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity rs is
  port (clr : in  std_logic;
        clk : in  std_logic;
        s   : in  std_logic;
        r   : in  std_logic;
        q   : out std_logic);
end rs;

architecture behavioral of rs is

  begin
    process (clr, clk)
    begin
      if (clr = '1') then
        q <= '0';
      elsif (rising_edge(clk)) then
        if (r = '1') then
          q <= '0';
        elsif (s = '1') then
          q <= '1';
        end if;
      end if;
    end process;
  end behavioral;
--------------------------------------------------------------------------------
--------------- Ceci est la description d'un comparateur 16 bit. ---------------
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity cmp is
  port (a   : in  std_logic_vector(15 downto 0);
        b   : in  std_logic_vector(15 downto 0);
        eq  : out std_logic);
end cmp;

architecture behavioral of cmp is

  begin
    eq <= '1' when (a = b) else
          '0';
  end behavioral;
--------------------------------------------------------------------------------
------------------- Ceci est la description d'une pwm 16 bit. ------------------
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity pwm is
  port (clr : in  std_logic;
        clk : in  std_logic;
        d   : in  std_logic_vector(15 downto 0);
        q   : out std_logic);
end pwm;

architecture behavioral of pwm is

  signal zero     : std_logic;
  signal eq       : std_logic;
  signal internal : std_logic_vector(15 downto 0);

  component cnt16
    port (clr : in  std_logic;
          clk : in  std_logic;
          q   : out std_logic_vector(15 downto 0));
  end component;

  component rs
    port (clr : in  std_logic;
          clk : in  std_logic;
          s   : in  std_logic;
          r   : in  std_logic;
          q   : out std_logic);
  end component;

  component cmp
    port (a   : in  std_logic_vector(15 downto 0);
          b   : in  std_logic_vector(15 downto 0);
          eq  : out std_logic);
  end component;

  begin
    counter  : cnt16 port map (clr, clk, internal);
    cmpzero  : cmp   port map (internal, x"0000", zero);
    cmplimit : cmp   port map (internal, d, eq);
    rsc      : rs    port map (clr, clk, zero, eq, q);
  end behavioral;
--------------------------------------------------------------------------------
-------------- Ceci est la description d'une quadruple pwm 16 bit. -------------
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity fourpwm is
  port (clr : in  std_logic;
        clk : in  std_logic;
        d0  : in  std_logic_vector(15 downto 0);
        q0  : out std_logic;
        d1  : in  std_logic_vector(15 downto 0);
        q1  : out std_logic;
        d2  : in  std_logic_vector(15 downto 0);
        q2  : out std_logic;
        d3  : in  std_logic_vector(15 downto 0);
        q3  : out std_logic);
end fourpwm;

architecture behavioral of fourpwm is

  signal zero     : std_logic;
  signal eq0      : std_logic;
  signal eq1      : std_logic;
  signal eq2      : std_logic;
  signal eq3      : std_logic;
  signal internal : std_logic_vector(15 downto 0);

  component cnt16
    port (clr : in  std_logic;
          clk : in  std_logic;
          q   : out std_logic_vector(15 downto 0));
  end component;

  component rs
    port (clr : in  std_logic;
          clk : in  std_logic;
          s   : in  std_logic;
          r   : in  std_logic;
          q   : out std_logic);
  end component;

  component cmp
    port (a   : in  std_logic_vector(15 downto 0);
          b   : in  std_logic_vector(15 downto 0);
          eq  : out std_logic);
  end component;

  begin
    counter   : cnt16 port map (clr, clk, internal);
    cmpzero   : cmp   port map (internal, x"0000", zero);
    cmplimit0 : cmp   port map (internal, d0, eq0);
    rsc0      : rs    port map (clr, clk, zero, eq0, q0);
    cmplimit1 : cmp   port map (internal, d1, eq1);
    rsc1      : rs    port map (clr, clk, zero, eq1, q1);
    cmplimit2 : cmp   port map (internal, d2, eq2);
    rsc2      : rs    port map (clr, clk, zero, eq2, q2);
    cmplimit3 : cmp   port map (internal, d3, eq3);
    rsc3      : rs    port map (clr, clk, zero, eq3, q3);
  end behavioral;
--------------------------------------------------------------------------------
